"""test Face class."""
from honeybee.typing import valid_string, valid_rad_string, valid_ep_string, \
    float_in_range, int_in_range, float_positive, int_positive, \
    tuple_with_length, list_with_length, normpath

import pytest


def test_valid_string():
    """Test the valid_string method."""
    correct_str = '0.5 in. Gypsum Wall'
    incorrect_str = '0.5 in., Gypsum Wall'
    long_str = 'This is an exceptionally long text string that should never be used ' \
        'for the name of anything in EnergyPlus for whatever reason'

    assert valid_string(correct_str) == '0.5in.GypsumWall'
    assert valid_string(incorrect_str) == '0.5in.GypsumWall'
    with pytest.raises(AssertionError):
        valid_string(long_str)


def test_valid_rad_string():
    """Test the valid_rad_string method."""
    correct_str = '0.5 in. Gypsum Wall'
    incorrect_str = '0.5 in., Gypsum Wall'
    long_str = 'This is an exceptionally long text string that should never be used ' \
        'for the name of anything in EnergyPlus but is actually ok for Radiance'

    assert valid_rad_string(correct_str) == '0.5in.GypsumWall'
    assert valid_rad_string(incorrect_str) == '0.5in.GypsumWall'
    valid_rad_string(long_str)


def test_valid_ep_string():
    """Test the valid_ep_string method."""
    correct_str = '1/2 in. Gypsum Board'
    incorrect_str = '1/2 in., Gypsum Board!'
    long_str = 'This is an exceptionally long text string that should never be used ' \
        'for the name of anything in EnergyPlus'

    assert valid_ep_string(correct_str) == correct_str
    assert valid_ep_string(incorrect_str) == correct_str
    with pytest.raises(AssertionError):
        valid_ep_string(long_str)


def test_float_in_range():
    """Test the float_in_range method."""
    assert isinstance(float_in_range(2.0, 0, 10, 'test number'), float)
    assert isinstance(float_in_range(2, 0, 10, 'test number'), float)
    assert isinstance(float_in_range('2', 0, 10, 'test number'), float)

    with pytest.raises(AssertionError):
        assert isinstance(float_in_range(2, 0, 1, 'test number'), float)
    with pytest.raises(TypeError):
        assert isinstance(float_in_range('two', 0, 10, 'test number'), float)
    with pytest.raises(TypeError):
        assert isinstance(float_in_range([2], 0, 10, 'test number'), float)

    try:
        float_in_range(2, 0, 1, 'test number')
    except AssertionError as e:
        assert 'test number' in str(e)


def test_int_in_range():
    """Test the float_in_range method."""
    assert isinstance(int_in_range(2.0, 0, 10), int)
    assert isinstance(int_in_range(2, 0, 10), int)
    assert isinstance(int_in_range('2', 0, 10), int)

    with pytest.raises(AssertionError):
        assert isinstance(int_in_range(2, 0, 1), float)
    with pytest.raises(TypeError):
        assert isinstance(int_in_range('two', 0, 10), float)
    with pytest.raises(TypeError):
        assert isinstance(int_in_range([2], 0, 10), float)

    try:
        int_in_range(2, 0, 1, 'test number')
    except AssertionError as e:
        assert 'test number' in str(e)


def test_float_positive():
    """Test the float_positive method."""
    assert isinstance(float_positive(2.0), float)
    assert isinstance(float_positive(2), float)
    assert isinstance(float_positive('2'), float)

    with pytest.raises(AssertionError):
        assert isinstance(float_positive(-2), float)
    with pytest.raises(TypeError):
        assert isinstance(float_positive('two'), float)
    with pytest.raises(TypeError):
        assert isinstance(float_positive([2]), float)

    try:
        float_positive(-2, 'test number')
    except AssertionError as e:
        assert 'test number' in str(e)


def test_int_positive():
    """Test the int_positive method."""
    assert isinstance(int_positive(2.0), int)
    assert isinstance(int_positive(2), int)
    assert isinstance(int_positive('2'), int)

    with pytest.raises(AssertionError):
        assert isinstance(int_positive(-2), float)
    with pytest.raises(TypeError):
        assert isinstance(int_positive('two'), float)
    with pytest.raises(TypeError):
        assert isinstance(int_positive([2]), float)

    try:
        int_positive(-2, 'test number')
    except AssertionError as e:
        assert 'test number' in str(e)


def test_tuple_with_length():
    """Test the tuple_with_length method."""
    assert isinstance(tuple_with_length((1, 2, 3), 3, float, 'test tuple'), tuple)
    assert isinstance(tuple_with_length([1, 2, 3], 3, float, 'test tuple'), tuple)
    assert isinstance(tuple_with_length(range(3), 3, float, 'test tuple'), tuple)
    assert isinstance(tuple_with_length((1.0, 2.0, 3.0), 3, float, 'test tuple'), tuple)
    assert isinstance(tuple_with_length(('1', '2', '3'), 3, float, 'test tuple'), tuple)

    with pytest.raises(AssertionError):
        tuple_with_length((1, 2, 3), 4, float, 'test tuple')
    with pytest.raises(TypeError):
        tuple_with_length(('one', 'two', 'three'), 3, float, 'test tuple')

    try:
        tuple_with_length((1, 2, 3), 4, float, 'test tuple')
    except AssertionError as e:
        assert 'test tuple' in str(e)


def test_list_with_length():
    """Test the list_with_length method."""
    assert isinstance(list_with_length((1, 2, 3), 3, float, 'test list'), list)
    assert isinstance(list_with_length([1, 2, 3], 3, float, 'test list'), list)
    assert isinstance(list_with_length(range(3), 3, float, 'test list'), list)
    assert isinstance(list_with_length((1.0, 2.0, 3.0), 3, float, 'test list'), list)
    assert isinstance(list_with_length(('1', '2', '3'), 3, float, 'test list'), list)

    with pytest.raises(AssertionError):
        list_with_length((1, 2, 3), 4, float, 'test list')
    with pytest.raises(TypeError):
        list_with_length(('one', 'two', 'three'), 3, float, 'test list')

    try:
        list_with_length((1, 2, 3), 4, float, 'test list')
    except AssertionError as e:
        assert 'test list' in str(e)



# coding: utf-8
"""A series of utility functions that are useful across several honeybee extensions."""


def model_extension_dicts(data, extension_key, room_ext_dicts=[], face_ext_dicts=[],
                          shade_ext_dicts=[], aperture_ext_dicts=[], door_ext_dicts=[]):
    """Get all Model property dictioanires of an extension organized by geometry type.

    Note that the order in which dictaionaries appear in the output lists is the
    same order as the geometry objects appear when requested from the model.
    For example, the shade_ext_dicts align with the model.shades.

    Args:
        data: A dictionary representation of an entire honeybee-core Model.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        room_ext_dicts: A list of Room extension property dictionaries that
            align with the serialized model.rooms.
        face_ext_dicts: A list of Face extension property dictionaries that
            align with the serialized model.faces.
        shade_ext_dicts: A list of Shade extension property dictionaries that
            align with the serialized model.shades.
        aperture_ext_dicts: A list of Aperture extension property dictionaries that
            align with the serialized model.apertures.
        door_ext_dicts: A list of Door extension property dictionaries that
            align with the serialized model.doors.
    """
    assert data['type'] == 'Model', \
        'Expected Model dictionary. Got {}.'.format(data['type'])

    # loop through the model dictionary using the same logic that the
    # model does when you requst rooms, faces, shades, apertures and doors.
    if 'rooms' in data:
        room_extension_dicts(data['rooms'], extension_key, room_ext_dicts,
                             face_ext_dicts, shade_ext_dicts, aperture_ext_dicts,
                             door_ext_dicts)
    if 'orphaned_faces' in data:
        face_extension_dicts(data['orphaned_faces'], extension_key, face_ext_dicts,
                             shade_ext_dicts, aperture_ext_dicts, door_ext_dicts)
    if 'orphaned_apertures' in data:
        aperture_extension_dicts(data['orphaned_apertures'], extension_key,
                                 aperture_ext_dicts, shade_ext_dicts)
    if 'orphaned_shades' in data:
        shade_extension_dicts(data['orphaned_shades'], extension_key, shade_ext_dicts)
    if 'orphaned_doors' in data:
        door_extension_dicts(data['orphaned_doors'], extension_key, door_ext_dicts)

    return room_ext_dicts, face_ext_dicts, shade_ext_dicts, \
        aperture_ext_dicts, door_ext_dicts


def room_extension_dicts(room_list, extension_key, room_ext_dicts=[], face_ext_dicts=[],
                         shade_ext_dicts=[], aperture_ext_dicts=[], door_ext_dicts=[]):
    """Get all Room property dictionaires of an extension organized by geometry type.

    Args:
        room_list: A list of Room dictionaries.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        room_ext_dicts: A list with the Room extension property dictionaries.
        face_ext_dicts: A list with Face extension property dictionaries.
        shade_ext_dicts: A list with Shade extension property dictionaries.
        aperture_ext_dicts: A list with Aperture extension property dictionaries.
        door_ext_dicts: A list with Door extension property dictionaries.
    """
    for room_dict in room_list:
        room_ext_dicts.append(room_dict['properties'][extension_key])
        if 'outdoor_shades' in room_dict:
            shade_extension_dicts(room_dict['outdoor_shades'], extension_key,
                                  shade_ext_dicts)
        if 'indoor_shades' in room_dict:
            shade_extension_dicts(room_dict['indoor_shades'], extension_key,
                                  shade_ext_dicts)
        face_extension_dicts(room_dict['faces'], extension_key, face_ext_dicts,
                             shade_ext_dicts, aperture_ext_dicts, door_ext_dicts)
    return room_ext_dicts, face_ext_dicts, shade_ext_dicts, \
        aperture_ext_dicts, door_ext_dicts


def face_extension_dicts(face_list, extension_key, face_ext_dicts=[],
                         shade_ext_dicts=[], aperture_ext_dicts=[], door_ext_dicts=[]):
    """Get all Face property dictionaires of an extension organized by geometry type.

    Args:
        face_list: A list of Room dictionaries.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        face_ext_dicts: A list with Face extension property dictionaries.
        shade_ext_dicts: A list with Shade extension property dictionaries.
        aperture_ext_dicts: A list with Aperture extension property dictionaries.
        door_ext_dicts: A list with Door extension property dictionaries.
    """
    for face_dict in face_list:
        face_ext_dicts.append(face_dict['properties'][extension_key])
        if 'outdoor_shades' in face_dict:
            shade_extension_dicts(face_dict['outdoor_shades'], extension_key,
                                  shade_ext_dicts)
        if 'indoor_shades' in face_dict:
            shade_extension_dicts(face_dict['indoor_shades'], extension_key,
                                  shade_ext_dicts)
        if 'apertures' in face_dict:
            aperture_extension_dicts(face_dict['apertures'], extension_key,
                                     aperture_ext_dicts, shade_ext_dicts)
        if 'doors' in face_dict:
            door_extension_dicts(face_dict['doors'], extension_key, door_ext_dicts)
    return face_ext_dicts, shade_ext_dicts, aperture_ext_dicts, door_ext_dicts


def shade_extension_dicts(shade_list, extension_key, shade_ext_dicts=[]):
    """Get all Shade property dictionaires of an extension organized by geometry type.

    Args:
        shade_list: A list of Shade dictionaries.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        shade_ext_dicts: A list with Shade extension property dictionaries.
    """
    for shd_dict in shade_list:
        shade_ext_dicts.append(shd_dict['properties'][extension_key])
    return shade_ext_dicts


def aperture_extension_dicts(aperture_list, extension_key, aperture_ext_dicts=[],
                             shade_ext_dicts=[]):
    """Get all Shade property dictionaires of an extension organized by geometry type.

    Args:
        shade_list: A list of Shade dictionaries.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        aperture_ext_dicts: A list with Aperture extension property dictionaries.
        shade_ext_dicts: A list with Shade extension property dictionaries.
    """
    for ap_dict in aperture_list:
        aperture_ext_dicts.append(ap_dict['properties'][extension_key])
        if 'outdoor_shades' in ap_dict:
            shade_extension_dicts(ap_dict['outdoor_shades'], extension_key, shade_ext_dicts)
        if 'indoor_shades' in ap_dict:
            shade_extension_dicts(ap_dict['indoor_shades'], extension_key, shade_ext_dicts)
    return aperture_ext_dicts, shade_ext_dicts


def door_extension_dicts(door_list, extension_key, door_ext_dicts=[]):
    """Get all Shade property dictionaires of an extension organized by geometry type.

    Args:
        shade_list: A list of Shade dictionaries.
        extension_key: Text for the key of the extension (eg. "energy", "radiance").

    Returns:
        door_ext_dicts: A list with Door extension property dictionaries.
    """
    for dr_dict in door_list:
        door_ext_dicts.append(dr_dict['properties'][extension_key])
    return door_ext_dicts



# coding: utf-8
"""Base class for all geometry objects."""
from .typing import valid_string


class _Base(object):
    """A base class for all geometry objects.

    Properties:
        name
        display_name
    """
    __slots__ = ('_name', '_display_name', '_properties')

    def __init__(self, name):
        """Initialize base object.

        Args:
            name: Object name. Must be < 100 characters.
        """
        self.name = name
        self._properties = None

    @property
    def name(self):
        """Get or set the object name (including only legal characters)."""
        return self._name

    @name.setter
    def name(self, value):
        self._name = valid_string(value, 'honeybee object name')
        self._display_name = value

    @property
    def display_name(self):
        """Get the original input name by user.

        If there are no illegal characters in name then name and display_name will
        be the same. Legal characters are ., A-Z, a-z, 0-9, _ and -.
        Invalid characters are automatically removed from the original name for
        compatability with simulation engines.
        """
        return self._display_name

    @property
    def properties(self):
        """Get object properties, including Radiance, Energy and other properties."""
        return self._properties

    def duplicate(self):
        """Get a copy of this object."""
        return self.__copy__()

    def __copy__(self):
        new_obj = self.__class__(self.name)
        new_obj._display_name = self.display_name
        return new_obj

    def ToString(self):
        return self.__repr__()

    def __repr__(self):
        return 'Honeybee Base Object: %s' % self.display_name
